import 'package:flutter/material.dart';
import 'dart:io';
import '../classes/addEventImage.dart';
import '../classes/alert.dart';
import 'package:flutter_dotenv/flutter_dotenv.dart';
import 'package:flutter_gemini/flutter_gemini.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:http/http.dart' as http;
import "ItemMenu.dart";
import 'UpdateEventImage.dart';

import 'dart:convert';

import '../variable/globals.dart' as globals;
import '../classes/MyDrawer.dart';

import '../classes/ad_helper.dart';
import 'package:google_mobile_ads/google_mobile_ads.dart';
import '../classes/auth_service.dart';

void main() async {
  WidgetsFlutterBinding.ensureInitialized();
  await MobileAds.instance.initialize();
  runApp(MyApp());
}

class MyApp extends StatelessWidget {
  const MyApp({Key? key}) : super(key: key);

  // This widget is the root of your application.

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      title: 'Flutter Demo',
      theme: ThemeData(
        // This is the theme of your application.
        //
        // Try running your application with "flutter run". You'll see the
        // application has a blue toolbar. Then, without quitting the app, try
        // changing the primarySwatch below to Colors.green and then invoke
        // "hot reload" (press "r" in the console where you ran "flutter run",
        // or simply save your changes to "hot reload" in a Flutter IDE).
        // Notice that the counter didn't reset back to zero; the application
        // is not restarted.
        primarySwatch: Colors.blue,
      ),
    );
  }
}

// A screen that allows users to take a picture using a given camera.
class DisplayPictureScreen extends StatefulWidget {
  final String imagePath;

  const DisplayPictureScreen({super.key, required this.imagePath});

  @override
  DisplayPictureScreenState createState() => DisplayPictureScreenState();
}

// A widget that displays the picture taken by the user.
class DisplayPictureScreenState extends State<DisplayPictureScreen>
    with ShowDescImageAdd, ShowAlertDialog, TickerProviderStateMixin {
  BannerAd? _bannerAd;
  final AuthService _authService = AuthService();

  late AnimationController controller;
  @override
  void initState() {
    super.initState();
    _authService.checkTokenStatus(context);

    AdHelper.createBannerAd(() => setState(() {})).then((ad) {
      setState(() {
        _bannerAd = ad;
      });
    });

    controller = AnimationController(
      /// [AnimationController]s can be created with `vsync: this` because of
      /// [TickerProviderStateMixin].
      vsync: this,
      duration: const Duration(seconds: 5),
    )..addListener(() {
        setState(() {});
      });
    controller.repeat(reverse: false);

    _getEventInfosFromImage();
  }

  @override
  void dispose() {
    controller.dispose();
    super.dispose();
  }

  Future<void> displayError(String e) async {
    print("problem gemini : ${e}");
    showAlertDialog(context, 'Error IA',
        "L'IA de Google n'a pas su analyser l'image. Recommecer avec une autre");
  }

  void _showErrorDialog(BuildContext context, String title, String message) {
    showAlertDialog(context, title, message);
  }

  Future<Map<String, dynamic>?> _fetchGeolocation(String place) async {
    final apiKey = dotenv.env['PLACE_API_KEY'] ?? '';
    final response = await http.get(Uri.parse(
        'https://maps.googleapis.com/maps/api/place/textsearch/json?query=${place}}&key=$apiKey'));

    if (response.statusCode == 200) {
      final data = json.decode(response.body);
      if (data['results'].isNotEmpty) {
        return data['results'][0]['geometry']['location'];
      }
    }
    return null;
  }

  Future<bool> _isDuplicateEvent(String accessToken,
      Map<String, dynamic> jsonData, Map<String, dynamic> location) async {
    final url = Uri.parse(
        "${globals.api}/events/search?item=${jsonData["name"]}&date_event=${jsonData["start_date"]}"
        "&min_lat=${location['lat']}&max_lat=${location['lat']}"
        "&min_lon=${location['lng']}&max_lon=${location['lng']}");

    final response = await http.get(url,
        headers: {HttpHeaders.cookieHeader: 'access_token=$accessToken'});

    if (response.statusCode == 200) {
      final events = jsonDecode(utf8.decode(response.bodyBytes));
      return events.isNotEmpty;
    }
    return false;
  }

  Future<void> searchEvents(String json, String imagePath) async {
    print(json.replaceAll("'''json", '').replaceAll("'''", ""));
    SharedPreferences prefs = await SharedPreferences.getInstance();
    try {
      Map<String, dynamic> jsonData =
          jsonDecode(json.replaceAll("```json", '').replaceAll("```", ""));
      print("json : ${jsonData}");
      var name = jsonData["name"];
      print("name : ${name}");
      var place = jsonData["place"];
      var date = jsonData["start_date"];
      var accessToken = prefs.getString("access_token") ?? "";

      if (accessToken.isNotEmpty) {
        final location = await _fetchGeolocation(place);
        if (location == null) {
          _showErrorDialog(
              context, "Erreur serveur", "Aucune donnée geographique");
          return;
        }

        final url = Uri.parse(
            "${globals.api}/events/search?item=${name}&date_event=${date}"
            "&min_lat=${location['lat']}&max_lat=${location['lat']}"
            "&min_lon=${location['lng']}&max_lon=${location['lng']}");

        final response = await http.get(url,
            headers: {HttpHeaders.cookieHeader: 'access_token=$accessToken'});
        if (response.statusCode == 200) {
          var events = jsonDecode(utf8.decode(response.bodyBytes));
          print("reponse  http : ${events.length}");
          if (events.length == 0) {
            Navigator.push(
                context,
                MaterialPageRoute(
                    builder: (_) => UpdateeventImage(
                        events: jsonData, imagePath: imagePath)));
          } else {
            Navigator.push(
                context,
                MaterialPageRoute(
                    builder: (_) => ItemMenu(title: events[0]["id"])));
          }
        } else {
          showAlertDialog(context, 'Erreur de reponse',
              "response status code update : ${response.statusCode}");
        }
      } else {
        showAlertDialog(context, "Erreur de reponse", "Erreur de token");
      }
    } catch (e) {
      showAlertDialog(
          context, "Erreur IA", "Erreur de format de donnée fourni par l'IA");
    }

    //showDescImageAddDialog(context, message);
  }

  Future<void> _getEventInfosFromImage() async {
    await dotenv.load();

    final gemini = Gemini.init(
        apiKey: dotenv.env['GEMINI_API_KEY']!, enableDebugging: true);

    final file = File(widget.imagePath);

    gemini
        .textAndImage(
            text:
                "Peux-tu donner le nom, la date (si l'année n'est pas précisé, mettez l'année actuelle ou future) et le lieu de l'évènement sous format JSON (sans le caratère json au début de la chaine de caractère) avec les valeurs suivantes : name, place, description, tags (tableau sans espace), organizers (tableau), start_date et end_date (si le end_date est vide, alors donnez une valeur de six de plus par rapport à start_date) sous le format en YYYY-MM-DD HH:mm:ssZ",
            images: [file.readAsBytesSync()],
            modelName: "models/gemini-1.5-pro-latest")
        .then((value) => searchEvents(
            value?.content?.parts?.last.text ?? '', widget.imagePath))
        .catchError((e) => displayError);
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        appBar: AppBar(title: const Text('Display the Picture')),
        // The image is stored as a file on the device. Use the `Image.file`
        // constructor with the given path to display the image.
        drawer: MyDrawer(),
        body: Padding(
            padding: const EdgeInsets.all(20.0),
            child: Column(
                mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                children: <Widget>[
                  _bannerAd == null
                      ? SizedBox.shrink()
                      : SizedBox(
                          height: _bannerAd!.size.height.toDouble(),
                          width: _bannerAd!.size.width.toDouble(),
                          child: AdWidget(ad: _bannerAd!)),
                  Text(
                    'Analyse de l\'image en cours',
                    style: Theme.of(context).textTheme.titleLarge,
                  ),
                  CircularProgressIndicator(
                    value: controller.value,
                    semanticsLabel: 'Loading progress',
                  ),
                ])));
  }
}
